import json
import urllib.parse

import schemeld


def parse_url(url: str | urllib.parse.ParseResult) -> urllib.parse.ParseResult:
    if isinstance(url, urllib.parse.ParseResult):
        return url
    return urllib.parse.urlparse(url)


def determine_concepts_internal(json: dict | list, context, outputs: list[schemeld.Concept]) -> None:
    if isinstance(json, list):
        for m in json:
            determine_concepts_internal(m, context, outputs)
        return

    assert isinstance(json, dict), type(json)
    context = parse_url(json.get('@context', context))
    assert context.netloc == 'schema.org'

    if '@graph' in json:
        determine_concepts_internal(json['@graph'], context, outputs)
    else:
        outputs.append(schemeld.Concept(context, json))


def determine_concepts(json: dict | list) -> list[schemeld.Concept]:
    concepts = []
    determine_concepts_internal(json, '', concepts)
    return concepts


def determine_concepts_in_soup(soup) -> list[schemeld.Concept]:
    # TODO: Check type
    ld_json_elements = soup.find_all('script', type='application/ld+json')
    concepts = []
    for e in ld_json_elements:
        json_data = json.loads(e.string)
        concepts.extend(determine_concepts(json_data))
    return concepts
